#include "common.cuh"
#include "lru_cache.h"



namespace py = pybind11;

PYBIND11_MODULE(TORCH_EXTENSION_NAME, m){


    py::class_<gpucache::CacheConfig> cfg(m,"CacheConfig");

    cfg.def_readwrite("strategy", &gpucache::CacheConfig::strategy)
       .def_readwrite("capacity",&gpucache::CacheConfig::capacity)
       .def_readwrite("key_size", &gpucache::CacheConfig::keySize)
       .def_readwrite("value_size",&gpucache::CacheConfig::valueSize)
       .def_readwrite("max_query_num", &gpucache::CacheConfig::maxQueryNum)
       .def_readwrite("device_id",&gpucache::CacheConfig::deviceId);

    py::enum_<gpucache::CacheConfig::CacheEvictStrategy>(cfg,"CacheEvictStrategy")
        .value("LRU",gpucache::CacheConfig::CacheEvictStrategy::LRU)
        .value("LFU",gpucache::CacheConfig::CacheEvictStrategy::LFU)
        .value("FIFO",gpucache::CacheConfig::CacheEvictStrategy::FIFO);

    py::class_<gpucache::LRUCacheWrapper> lru_cache(m, "LRUCache");

    lru_cache
        .def(py::init<at::Tensor, gpucache::CacheConfig>())
        .def("Get",&gpucache::LRUCacheWrapper::Get,"get values for keys,  find_mask return whether each key exists in cache")
        .def("Put",&gpucache::LRUCacheWrapper::Put,"put key-value pairs")
        .def("Strategy",&gpucache::LRUCacheWrapper::Strategy,"get evict strategy")
        .def("Capacity",&gpucache::LRUCacheWrapper::Capacity,"return cache capacity")
        .def("KeySize",&gpucache::LRUCacheWrapper::KeySize,"return key size")
        .def("ValueSize",&gpucache::LRUCacheWrapper::ValueSize,"return value size")
        .def("MaxQueryNum",&gpucache::LRUCacheWrapper::MaxQueryNum,"return max number of keys to get or key-values to put once")
        .def("Clear",&gpucache::LRUCacheWrapper::Clear,"clear cache")
        .def("Device",&gpucache::LRUCacheWrapper::DeviceId,"return device id");

    m.def("NewLRUCache", &gpucache::NewLRUCache, "create a lru cache");
    
}


