namespace gpucache {
    struct CacheConfig {
        enum CacheEvictStrategy {
            FIFO,
            LRU,
            LFU
        };
        CacheEvictStrategy strategy;
        uint64_t capacity;
        uint32_t keySize;
        uint32_t valueSize;
        uint32_t maxQueryNum;
        uint64_t deviceId;
    };

    

    class LRUCacheWrapper{
        public:
        LRUCacheWrapper(at::Tensor t, CacheConfig cfg);
        ~LRUCacheWrapper();
        void Get(uint32_t num_query, void *queries, void* result, bool* find_mask);
        void Put(uint32_t num_query, void* put_keys, void* put_values, uint32_t* n_evict, void* evict_keys);
        CacheConfig::CacheEvictStrategy Strategy;
        uint64_t Capacity();
        uint32_t KeySize();
        uint32_t ValueSize();
        uint32_t MaxQueryNum();
        uint64_t DeviceId();
        void Clear();


        private:
            void* lru_cache;
            c10::ScalarType dtype;
            bool* key_is_uint32;
            CacheConfig cache_cfg;
    };
    
    LRUCacheWrapper NewLRUCache(at::Tensor t, CacheConfig cfg){
        return LRUCacheWrapper(t,cfg);
    }
}